#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <signal.h>


int getbaud(int fd) 
{
	struct termios termAttr;
	int inputSpeed = -1;
	speed_t baudRate;
	tcgetattr(fd, &termAttr);
	/* Get the input speed.                              */
	baudRate = cfgetispeed(&termAttr);
	switch (baudRate) {
		case B0:      inputSpeed = 0; break;
		case B50:     inputSpeed = 50; break;
		case B110:    inputSpeed = 110; break;
		case B134:    inputSpeed = 134; break;
		case B150:    inputSpeed = 150; break;
		case B200:    inputSpeed = 200; break;
		case B300:    inputSpeed = 300; break;
		case B600:    inputSpeed = 600; break;
		case B1200:   inputSpeed = 1200; break;
		case B1800:   inputSpeed = 1800; break;
		case B2400:   inputSpeed = 2400; break;
		case B4800:   inputSpeed = 4800; break;
		case B9600:   inputSpeed = 9600; break;
		case B19200:  inputSpeed = 19200; break;
		case B38400:  inputSpeed = 38400; break;
		case B115200:  inputSpeed = 115200; break;
	}
	return inputSpeed;
}


int fd;

int initport(int fd) 
{
	struct termios options;
	// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 115200...
	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);

	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;

	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);
	return 1;
}


void at_exit() 
{
  close(fd);
  printf("ended");
}


void termination_handler (int signum)
{
  at_exit();
  //exit(0);
  return;//(0);
}

     
int handle_signals()
{
	struct sigaction new_action, old_action;

	/* Set up the structure to specify the new action. */
	new_action.sa_handler = termination_handler;
	sigemptyset (&new_action.sa_mask);
	new_action.sa_flags = 0;
	sigaction (SIGINT, NULL, &old_action);
	if (old_action.sa_handler != SIG_IGN)
	    sigaction (SIGINT, &new_action, NULL);
	sigaction (SIGHUP, NULL, &old_action);
	if (old_action.sa_handler != SIG_IGN)
	    sigaction (SIGHUP, &new_action, NULL);
	sigaction (SIGTERM, NULL, &old_action);
	if (old_action.sa_handler != SIG_IGN)
	    sigaction (SIGTERM, &new_action, NULL);
}


void usage() 
{
	printf("Use it like this:\n");
	printf("cat ../TomCat.flash | ./germs_up /dev/ttyS0\n\n");
	printf("The .flash file must be in the directory above.\n");
	printf("Be shure the file was created in DOS format, c\n");
	printf("otherwise you have to make an unix2dos conversion before.\n");
}


int main(int argc, char **argv) 
{
	int  i=0, r;
        char ch=0;
	char str[1025];
	int timeout=0, timer=0;
        int count=0, retry=0;

	//If no arguments are found, print help
	if(argc<2) {
	  usage();
	  return 1;
	}

	//open serial port
	handle_signals();
	fd = open(argv[1], O_RDWR | O_NOCTTY | O_SYNC );  //| O_NDELAY); // use O_NDELAY for non blocking);
	//atexit(at_exit);

	if (fd == -1)
	 {
	  printf("open_port: Unable to open %s\n", argv[1]);
          return 1;
	 }
	 else
	 {
	  //atexit(at_exit);
          fcntl(fd, F_SETFL, 0);
	 }
	
	//initialize serial port
	printf("baud=%d\n", getbaud(fd));
	initport(fd);
	printf("baud=%d\n", getbaud(fd));


        printf("starting transmission...\n");
	count = 1;

	//read line from file
	while(fgets(str, 1024, stdin))
	{
		if(strstr(str, "#")) continue;		//skip comments
	  
		printf("sending line %d\n->", count);

		//send string byte after byte
		for(i=0;i < strlen(str);i++)
		{
        		
			usleep(2000);			//wait 2 milliseconds bevor sending next character
			write(fd, &str[i], 1);
			printf("%c",str[i]);
        	}

        	timer=0; timeout=50000; ch=0; retry=0;
		printf("\nwaiting...");
		
		//wait for response from DSO (prompt is +)
        	while(ch != '+')
		{
			//r = read(fd, &ch, 1);
			read(fd, &ch, 1);

			timer++;
			//printf("\rwaiting... (timer %d) ->%c", timer, ch);
			//printf("\rwaiting... (timer %d) returncode ->%d", timer, r);
			printf("\rwaiting... (timer %d)", timer);

			//timeout! Normaly after about 50 to 200 tries the response should come
			//but sometimes DSO is thinking a little bit longer about the last instruction
			if (timer >= timeout || ch == '?')
			{
				timer=0; timeout=5000; 
				if(ch == '?')
				{
					printf("\nW2000A did not understand, sending again...\n->");
				}
				else
				{
					printf("\ntimeout error, sending again...\n->");
					retry++;
					timeout *= retry;
				}
				//send the string again with fallback timing
				for(i=0; i<strlen(str);i++)
				{
					usleep(10000); //wait 10 milliseconds
					write(fd, &str[i], 1);					
					printf("%c",str[i]);
				}
				
			}
			if(retry > 10)		// 10 retries should be enough
			{
				printf("\nsend error -> terminating\n");
				return(1);	
			}

        	}
		
		//print the last received character (should be the prompt)
		printf("\nresponse from W2000A is %c\n",ch);
		count++;
        	
        	fflush(stdout);
	}
        printf("\nending transmission...\n");
	close(fd);
        printf("\nall done\n");
	return 0;
}

